#!/usr/bin/env python

import os, popen2, tempfile

def decrypt(data, cert, privkey):
	msg, err = _runCommand("openssl smime -decrypt -inform PEM -recip %s -inkey %s" % \
							(cert,privkey), data)
	if err != '':
		raise Exception('openssl failed: %s' % err)
	return msg

def encrypt(data, cert):
	msg, err = _runCommand("openssl smime -encrypt -des3 -outform PEM %s" % cert, data)
	if err != '':
		raise Exception('openssl failed: %s' % err)
	return msg

def encodeFileB64(inp, out):
	_, err = _runCommand("openssl base64 -in %s -out %s" % (inp, out), "")
	if err != '':
		raise Exception('openssl failed: %s' % err)
	
def decodeFileB64(inp, out):
	_, err = _runCommand("openssl base64 -d -in %s -out %s" % (inp, out), "")
	if err != '':
		raise Exception('openssl failed: %s' % err)
			
def signFile(inp, out, cert, privkey):
	b64 = tempfile.mktemp()
	try:
		encodeFileB64(inp, b64)
		_, err = _runCommand("openssl smime -sign -signer %s -inkey %s -in %s -out %s" % \
						 (cert, privkey, b64, out), "")
		if err != '':
			raise Exception('openssl failed: %s' % err)
	finally:
		try: os.remove(b64)
		except:	pass
	
def verifyFile(inp, out, certs):
	"""Input File to verify, Output File with verified message, Cert directory"""
	signer = tempfile.mktemp()
	b64 = tempfile.mktemp()
	try:
		_, err = _runCommand("openssl smime -verify -CApath %s -signer %s -out %s -in %s" % \
							 (certs, signer, b64, inp), "")
		if err != 'Verification successful\n':
			raise Exception('openssl failed: %s' % err)
		decodeFileB64(b64, out)
		signerCert = "".join(open(signer,'r').readlines())
	finally:
		try: os.remove(signer)
		except:	pass
		try: os.remove(b64)
		except:	pass
		
	return signerCert
	
def x509subject(cert):
	msg, err = _runCommand("openssl x509 -subject -noout", cert)
	if err != '':
		raise Exception('openssl failed: %s' % err)
	return msg.replace("subject= ","",1).strip()

def _mkdtemp(directory = "."):
	directory = os.path.abspath(directory)
	out, err = _runCommand('/bin/mktemp -d %s/.XXXXXX' % directory, "")
	if err != '':
		raise Exception('mktemp failed: %s' % err)
	return os.path.abspath(out.strip())
	
def _runCommand(cmd, data):
	p = popen2.Popen3(cmd,1)
	p.tochild.write(data)
	p.tochild.close()
	tmp = p.fromchild.readlines()
	p.fromchild.close()
	err = p.childerr.readlines()
	p.childerr.close()
	p.wait()
	return (''.join(tmp), ''.join(err))
	
	
if __name__ == '__main__':
	import time
	s = "This is the text to be transmitted! "
	start = time.time()
	print "Text          :", repr(s)
	print "Encrypted Text:",repr(encrypt(s, "cert.pem")),"... and more"
	print "Decrypted Text:",repr(decrypt(encrypt(s, "cert.pem"), "cert.pem", "privkey.pem"))
	print "needed "+str(time.time()-start)+" seconds"
