"""Abstracts a Test

This module uses a message and forces it to have the attribute 'name'.

Author: Thomas Low <thomas.low@cern.ch>
"""

import os, time, random, message, re

name_pattern = re.compile(r"^[a-zA-Z][\w]*$")

class Test(message.Message):
	"""A message with a mandatory attribute 'name'.
	
	NAME has to match ^[a-zA-Z][\w]*$
	The test also contains always an attribute "id" which is unique for	each 
	test-class.
	"""
	def __init__(self, name):
		"""Inits the class with a "name" and an "id" attribute."""
		message.Message.__init__(self)
		if not name_pattern.match(str(name)):
			raise ValueError("test name has to match ^[a-zA-Z][\w]*$")
		setattr(self, 'testName', name)
		setattr(self, 'testId', _getUniqueId())
		
	def __setattr__(self, name, value):
		"""Sets an attribute of the test."""
		if str(name) == 'testName' and not name_pattern.match(str(value)):
			raise ValueError("test name has to match ^[a-zA-Z][\w]*$")
		message.Message.__setattr__(self, name, value)

def stringToTest(s):
	"""Converts a tring into a Test.
	
	Raises AttributeError if string has no "name" attribute.
	"""
	m = message.stringToMessage(s)
	t = Test(m.testName)
	t.update(message.messageToDict(m))
	return t
	
def testToString(t):
	"""Converts a Test into a string."""
	return str(t)
	
def dictToTest(d):
	"""Converts a dictionary into a Test.
	
	Raises a AttributeError if the dict has no attribute "name".
	"""
	m = message.dictToMessage(d)
	t = Test(m.testName)
	t.update(d)
	return t

def testToDict(t):
	"""Converts a Test into a dictionary."""
	return message.messageToDict(t)
	
def _getUniqueId():
	"""Returns an hopefully unique string."""
	fp = str(random.random())
	linux = "echo -n "+fp+repr(time.time())+" | md5sum |  sed 's/ .*//'"
	return os.popen(linux).readline().split()[0]

