"""rest.py"""

import os, logging, re, urllib
from pyamq.lib import broker, util

log = logging.getLogger(__name__)

destination_pattern=re.compile('/(.+)/(.+)')
response_pattern=re.compile('HTTP/\S+ (\d+) OK')

class RestBroker(broker.Broker):

	def __init__(self, info, handler):
	
		self.receipt = 0
	
		if not info['port']:
			info['port'] = 80
		
		self.url = info['protocol']+"://"+info['host']+":"+str(info['port'])+str(info['path'])
		
		proxy_url = os.environ.get('http_proxy',os.environ.get('HTTP_PROXY'))
		if proxy_url:
			log.warning('Using http_proxy=%s' % proxy_url)
			try:
				#override info with the proxy information
				d = util.getUrlAsDict(proxy_url)
				if not d['port']:
					d['port'] = 80
				info = d
			except Exception, err:
				log.error("proxy url not valid: %s " % proxy_url)
			
		broker.Broker.__init__(self, info, handler)
	
	
	def doConnect(self):
		self._onConnected()
		return ""
		
	
	def doDisconnect(self):
		return ""
		
	
	def doSubscribe(self, destination):
		raise NotImplementedError("Subscribing with REST is not implemented")
	
	
	def doUnsubscribe(self, destination):
		raise NotImplementedError("Subscribing with REST is not implemented")
	
	
	def decodeMessage(self, data):
		pos = data.find('\r\n\r\n')
		if pos > -1:
			lines = data.split("\r\n")
			#print lines[0].strip()
			if response_pattern.match(lines[0].strip()) and self.receipt:
				self.receipt = 0
				self._onMsgSuccess(0)
				self.kill(1)
			else:
				log.error("Wrong HTTP response: %s" % repr(data[:30]))
				self.kill()
			return len(data)
		else:
			return 0
	
							
	def encodeMessage(self, msg):
		#print msg
		try:
			d = msg['header'].copy()
		except:
			d = {}
			
		
			
		t, dest = destination_pattern.match(d['destination']).groups()
		
		d['type'] = t
		d['destination'] = dest
		d['body'] = msg['body']
		
		body = urllib.urlencode(d)
		
		data =	'POST %s HTTP/1.1\r\n'\
				'Host: %s\r\n'\
				'Content-Type: application/x-www-form-urlencoded\r\n'\
				'Content-Length: %d\r\n\r\n' % \
				(self.url, self.info['host'], len(body))
		data += body
		
		self.receipt = 1
		return data
