"""stomp.py"""

import logging
from pyamq.lib import broker

log = logging.getLogger(__name__)

class StompBroker(broker.Broker):

	def __init__(self, info, handler):
		self.sessionid = None
		self.receiptid = 0
		
		if not info['port']:
			info['port'] = 61613
		
		broker.Broker.__init__(self, info, handler)
	
	
	def doConnect(self):
		return "CONNECT\n\n\0\n"
	
	
	def doDisconnect(self):
		return "DISCONNECT\n\n\0\n"
		
	
	def doSubscribe(self, destination):
		self.receiptid += 1
		return "SUBSCRIBE\ndestination:"+destination+"\nreceipt:"+self.sessionid+"-"+str(self.receiptid)+"\n\n\0\n"
	
	
	def doUnsubscribe(self, destination):
		return "UNSUBSCRIBE\ndestination:"+destination+"\n\n\0\n"
	
	
	def decodeMessage(self, data):
		pos = data.find('\0\n')
		count = 0
		while pos != -1:
			msg = data[:pos]	
			if msg[:9] == 'CONNECTED':
				#connect answer received, check validity
				startpos = msg.find('session:ID:')+len('session:ID:')
				endpos = msg.find('\n',startpos)
				if startpos > len('session:ID:') and endpos > startpos:
					self.sessionid = msg[startpos:endpos]
					self._onConnected() #signal valid connection
				else:
					log.warning(str(self)+" Stomp: connected-message corrupt")
					self.kill()
			
			elif msg[:7] == 'RECEIPT':
				#receipt answer received, check validity
				startpos = msg.find('receipt-id:')+len('receipt-id:')
				endpos = msg.find('\n',startpos)
				if startpos > len('receipt-id:') and endpos > startpos \
					and msg[startpos:endpos] == self.sessionid+"-"+str(self.receiptid):
					if self.subscription:
						self._onSubscribeSuccess()
					else:
						self._onMsgSuccess()
				else:
					print msg
					log.warning(str(self)+" receipt-message corrupt")
					self.kill()
		
			elif msg[:7] == 'MESSAGE':
				#message received, check validity!
				try:
					self._onMsgReceive(StompMessage(msg).getAsDict())
				except Exception, err:
					log.warning(str(self)+" stomp-message corrupt: %s" % err)
					self.kill()
			
			else:
				log.warning(str(self)+" unknown message: "+repr(self.read_buffer[:30]))
				self.kill()
			count += pos+2
			data = data[pos+2:]
			pos = data.find('\0\n')
		return count
	
							
	def encodeMessage(self, msg):
		self.receiptid += 1
		try:
			header = msg['header'].copy()
		except:
			header = {}
		header['content-length'] = len(str(msg['body']))
		header['receipt'] = self.sessionid+"-"+str(self.receiptid)
		data = 'SEND\ndestination:%s\n%s\n%s\0\n' % \
			(header['destination'],
			''.join(['%s:%s\n'%(k,str(v)) for (k,v) in header.items() if k != 'destination']),
			str(msg['body']))
		
		return data


class StompMessage:
	def __init__(self, msg):
		self.header = {}
		self.body = None
		self.plain = None
		if msg:
			self._readFromString(str(msg))
			
	def _readFromString(self, msg):
		header_begin = msg.find("\n")+1
		header_end = msg.find("\n\n")
		header = msg[header_begin:header_end]
		
		self.plain = msg
		self.body = msg[header_end+2:]
				
		for line in header.split("\n"):
			tuples = line.split(":",1)
			if len(tuples) == 2:
				self.header[tuples[0].strip()] = tuples[1].strip()
			else:
				raise Exception("Not a valid StompMessage")
				
	def getAsDict(self):
		return {'header': self.header, 'body': self.body}

	def __str__(self):
		return repr({'header': self.header, 'body': self.body})
